/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 * @class Contains all the functions necessary for library manipulation.
 */
var Library = function(){
	var config = {
		currentBookId: '',
		shelfId: '',
		open: false,
		libraryView: '',
		XUL_NS: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",
		XHTML_NS: "http://www.w3.org/1999/xhtml",
		bookMetadata: [],
		library_type: "regular",
		filter_text: "",
		filter_on: "title",
		bulk_delete: false,
		bulk_index: false,
		bulk_index_uncheck_event: false,
	}
	
    var treeView = function(){
		var self = this;
		if (config.filter_text == "") {
			//show all of them
			var bookMetadata = config.bookMetadata.slice(0);
		} else {
			//filter out the ones we want to display
			var bookMetadata = [];
			if (config.filter_text) {
				config.bookMetadata.forEach(function(element) {
					//we'll match on every property
					for (var i in element) {
						if ( i == config.filter_on) {
							var md_value = element[i];
							if(config.filter_on === 'created_date') {
								var d = new Date(+element[i]);
								md_value = d.toLocaleDateString();
							} 
							var filter = Library.prepareForComparison(config.filter_text);
							if (Library.prepareForComparison(md_value).indexOf(filter) != -1) {
								bookMetadata.push(element);
								break;
							}
						}
					}
				});
			} else {
				var bookMetadata = config.bookMetadata.slice(0);
			}
		}
		
		for (var i=0; i<bookMetadata.length; i++) {
			bookMetadata[i]["indexed"] = true;
			if (!INDEXER.existsInIndex(bookMetadata[i]["id"])) {
				bookMetadata[i]["indexed"] = false;
			}
			if (config.bulk_index) {
				bookMetadata[i]["index_search"] = config.bulk_index;
			} else if (!bookMetadata[i]["indexed"]) {
				bookMetadata[i]["index_search"] = config.bulk_index;
			} else if (bookMetadata[i]["indexed"] && !config.bulk_index_uncheck_event) {
				bookMetadata[i]["index_search"] = true;
			} else if (bookMetadata[i]["indexed"] && config.bulk_index_uncheck_event) {
				bookMetadata[i]["index_search"] = config.bulk_index;
			}
			
			bookMetadata[i]["col_del"] = config.bulk_delete;
		}
		
		config.bulk_index_uncheck_event = false;
		
        this.rowCount = bookMetadata.length;
        this.getCellText = function(row, col){
			try {
				if(col.id === 'created_date') {
					var d = new Date(+bookMetadata[row][col.id]);
					var datestr  = new XDate(parseInt(bookMetadata[row][col.id])).toString("yyyy-MM-dd");
					return datestr;
				} else if (col.id == "col_del"){
					
				} else {
					return  bookMetadata[row][col.id];
				}
			} catch(e) {
				return "";
			}
        };
        this.getCellValue = function(row, col){ 
			if (col.type == 2) {
				return bookMetadata[row][col.id];
			}
		};
        this.setTree = function(treebox){ this.treebox = treebox; };
        this.isEditable = function(row, col){ return col.editable; };
		this.setCellValue = function(row, col, value){ 
				bookMetadata[row][col.id] = value;
				return false;
		};
        this.isContainer = function(row){ return false; };
        this.isSeparator = function(row){ return false; };
        this.isSorted = function(){ return false; };
        this.getLevel = function(row){ return 0; };
        this.getImageSrc = function(row, col){ return null; };
		this.getRowProperties = function(row, prop){
			if (config.library_type != "ACFS") { 
				var isindexed = bookMetadata[row]["indexed"];
				if (isindexed) {
					return "book-indexed-moz";
				}
				return;
			}
			var bookstate = bookMetadata[row]["bookstate"];
			if (bookstate == "READYFORDOWNLOAD"){
				return "readyfordownload-moz";
			} else if (bookstate == "EXPIRED") {
				return "expired-moz";
			} else if (bookstate == "INACTIVE"){
				return "inactive-moz";
			}  else {
				return "downloadedbook-moz";
			}
		};
        this.getCellProperties = function(row, col, props){
		};
        this.getColumnProperties = function(colid, col, props){};
        this.cycleHeader = function(col, elem){};
    };
	
	/**
	 * Remove all the books from the variable
	 */
	var clear = function(){
		config.bookMetadata.splice(0, config.bookMetadata.length);
	}
	
	/**
	 * Initilize the library.
	 */
	var init = function(){
		var self = this;
		
		//Remove all the books from library which is used to display the library.
		self.clear();
		//Just the following variable will make the library to populate.
		config.bookMetadata = BookMetadata.getAll();
		//set the shelf description at the top of library.
		self.showShelfDescription('All Books', 'Contains all purchased or uploaded books.');
		//Sort the books in the library
		self.sort();
		window.resizeTo(UserPref.config.openWidth, UserPref.config.openHeight);
	}
	
	/**
	 * Sets the library view. table|thumbnail.
	 */
	var setLibraryView = function(){
		if (config.library_type == "ACFS") {
			//$("#index_search").hide();
			$("#col_del").hide();
			Library.setPubLibraryView();
			return false;
		}
			
		$("#index_search").show();
		$("#col_del").show();
		
		
		if(config.libraryView == 'table') {
			document.getElementById('library-azd').view = new treeView();
		} else if(config.libraryView == 'thumbnail'){
			$('#show_thumbnail_view').html('');
			
			if(config.bookMetadata.length == 0) {
				//If library is empty, then display message "No books available"
				var emptyDiv = document.createElementNS(config.XHTML_NS, "html:div");
				emptyDiv.setAttribute('class', 'no-thumbnail');
				var emptyP = document.createElementNS(config.XHTML_NS, "html:p");
				var emptyText = document.createTextNode('No books available');
				emptyP.appendChild(emptyText);
				emptyDiv.appendChild(emptyP);
				document.getElementById('show_thumbnail_view').appendChild(emptyDiv);
			} else{
				var bookMetadata = [];
				if (config.filter_text) {
					config.bookMetadata.forEach(function(element) {
						//we'll match on every property
						for (var i in element) {
							if ( i == config.filter_on) {
								var md_value = element[i];
								if(config.filter_on === 'created_date') {
									var d = new Date(+element[i]);
									md_value = d.toLocaleDateString();
								} 
								var filter = Library.prepareForComparison(config.filter_text);
								if (Library.prepareForComparison(md_value).indexOf(filter) != -1) {
									bookMetadata.push(element);
									break;
								}
							}
						}
					});
				} else {
					var bookMetadata = config.bookMetadata.slice(0);
				}
				
				//Show the books in their own container, with buttons and cover image
				for (var i = 0; i < bookMetadata.length; i++) {
					var bookId = bookMetadata[i]['id'];
					var bookTitle = bookMetadata[i]['title'];
					var bookAuthor = bookMetadata[i]['author'];
					var bookCover = bookMetadata[i]['cover_image'];
					var cover_type = bookMetadata[i]['cover_type'];
					var indexed = true;
					if (!INDEXER.existsInIndex(bookMetadata[i]["id"])) {
						indexed = false;
					}
					
					var container = document.createElementNS(config.XHTML_NS, "html:div");
					if(cover_type == 'image') {
						var clval = "thumbnail-item-image";
						if (indexed) clval = clval + " " + "indexed"; 
						container.setAttribute('class', clval);
					} else if(cover_type == 'avatar') {
						var clval = "thumbnail-item-avatar";
						if (indexed) clval = clval + " " + "indexed"; 
						container.setAttribute('class', clval);
					}
					
					container.style.backgroundImage = "url("+bookCover+")";
					container.setAttribute('ondragstart', 'event.dataTransfer.setData("text/plain", "'+bookId+'")');
					container.setAttribute('tooltiptext', bookTitle);
					container.setAttribute('onclick', 'javascript:AME.openBook("'+bookId+'")');
					
					var details = document.createElementNS(config.XHTML_NS, "html:div");
					details.setAttribute('class', 'thumb-details');
					
					var bookName = document.createElementNS(config.XHTML_NS, "html:p");
					var text1 = document.createTextNode(bookTitle);
					bookName.appendChild(text1);
					details.appendChild(bookName);
					
					var author = document.createElementNS(config.XHTML_NS, "html:p");
					var text2 = document.createTextNode(bookAuthor);
					author.appendChild(text2);
					details.appendChild(author);
					
					container.appendChild(details);
					
					var btn_container = document.createElementNS(config.XHTML_NS, "html:div");
					btn_container.setAttribute('class', 'thumb-buttons');
					
					var btn1 = document.createElementNS(config.XHTML_NS, "html:button");
					btn1.setAttribute('type', 'button');
					btn1.setAttribute('class', 'thumbnail-delete-book');
					btn1.setAttribute('style', 'width: 40px; height: 24px;');
					btn1.setAttribute('onclick', 'javascript: AME.deleteBook("'+bookId+'");event.stopPropagation();');
					var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
					btnImg.setAttribute('src', 'icons/ico_delete-book_18.png')
					btn1.appendChild(btnImg);
					btn_container.appendChild(btn1);
					
					var btn2 = document.createElementNS(config.XHTML_NS, "html:button");
					btn2.setAttribute('type', 'button');
					btn1.setAttribute('class', 'thumbnail-book-info');
					btn2.setAttribute('style', 'width: 40px; height: 24px;');
					btn2.setAttribute('onclick', 'javascript: AME.showBookInfoDialog("'+bookId+'");event.stopPropagation();');
					var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
					btnImg.setAttribute('src', 'icons/ico_book-info_18.png')
					btn2.appendChild(btnImg);
					btn_container.appendChild(btn2);
					
					container.appendChild(btn_container);
					
					document.getElementById('show_thumbnail_view').appendChild(container);
				}
			}
		}
		else if(config.libraryView == 'stack'){
			//	TODO: Implement library view in stack mode.
		}
	}
	
	/**
	 * Sets the library view. table|thumbnail.
	 */
	var setPubLibraryView = function(){
		//$("#index_search").hide();
		$("#col_del").hide();
			
		try {
		if(config.libraryView == 'table') {
			document.getElementById('library-azd').view = new treeView();
		} else if(config.libraryView == 'thumbnail'){
			$('#show_thumbnail_view').html('');
			
			if(config.bookMetadata.length == 0) {
				//If library is empty, then display message "No books available"
				var emptyDiv = document.createElementNS(config.XHTML_NS, "html:div");
				emptyDiv.setAttribute('class', 'no-thumbnail');
				var emptyP = document.createElementNS(config.XHTML_NS, "html:p");
				var emptyText = document.createTextNode('No books available');
				emptyP.appendChild(emptyText);
				emptyDiv.appendChild(emptyP);
				document.getElementById('show_thumbnail_view').appendChild(emptyDiv);
			} else{
				if (config.filter_text == "") {
					//show all of them
					var bookMetadata = config.bookMetadata.slice(0);
				} else {
					//filter out the ones we want to display
					var bookMetadata = [];
					if (config.filter_text) {
						config.bookMetadata.forEach(function(element) {
							//we'll match on every property
							for (var i in element) {
								if ( i == config.filter_on) {
									var md_value = element[i];
									if(config.filter_on === 'created_date') {
										var d = new Date(+element[i]);
										md_value = d.toLocaleDateString();
									} 
									var filter = Library.prepareForComparison(config.filter_text);
									if (Library.prepareForComparison(md_value).indexOf(filter) != -1) {
										bookMetadata.push(element);
										break;
									}
								}
							}
						});
					} else {
						var bookMetadata = config.bookMetadata.slice(0);
					}
				}
				//Show the books in their own container, with buttons and cover image
				for (var i = 0; i < bookMetadata.length; i++) {
					var bookId = bookMetadata[i]['id'];
					var bookTitle = bookMetadata[i]['title'];
					var bookAuthor = bookMetadata[i]['author'];
					var bookCover = bookMetadata[i]['cover_image'];
					var cover_type = bookMetadata[i]['cover_type'];
					var bookstate = bookMetadata[i]['bookstate'];
					var book_agreement = Library.config.bookMetadata[i]['agreement'];
					
					var container = document.createElementNS(config.XHTML_NS, "html:div");
					if(cover_type == 'image') {
						if (bookstate == "READYFORDOWNLOAD") {
							container.setAttribute('class', 'thumbnail-item-image for-download');
						} else if (bookstate == "EXPIRED") {
							container.setAttribute('class', 'thumbnail-item-image for-expired');
						} else if (bookstate == "INACTIVE") {
							container.setAttribute('class', 'thumbnail-item-image for-inactive');
						} else {
							container.setAttribute('class', 'thumbnail-item-image for-read');
						}
					} else if(cover_type == 'avatar') {	
						if (bookstate == "READYFORDOWNLOAD") {
							container.setAttribute('class', 'thumbnail-item-avatar for-download');
						}  else if (bookstate == "EXPIRED") {
							container.setAttribute('class', 'thumbnail-item-image for-expired');
						} else if (bookstate == "INACTIVE") {
							container.setAttribute('class', 'thumbnail-item-image for-inactive');
						} else {
							container.setAttribute('class', 'thumbnail-item-avatar for-read');
						}
					}
					
					container.style.backgroundImage = "url("+bookCover+")";
					//container.setAttribute('ondragstart', 'event.dataTransfer.setData("text/plain", "'+bookId+'")');
					container.setAttribute('tooltiptext', bookTitle);
					if (bookstate == "INACTIVE" || bookstate == "EXPIRED" || bookstate == "READYFORDOWNLOAD") {
						container.setAttribute('onclick', 'javascript:AME.showBookClickExceptionMsg("'+book_agreement+'",'+ '"'+ bookstate + '",'  + '",' + bookTitle + '"' +')');
					} else {
						container.setAttribute('onclick', 'javascript:AME.openBook("'+bookId+'")');
					}
					
					var details = document.createElementNS(config.XHTML_NS, "html:div");
					details.setAttribute('class', 'thumb-details');
					
					var bookName = document.createElementNS(config.XHTML_NS, "html:p");
					var text1 = document.createTextNode(bookTitle);
					bookName.appendChild(text1);
					details.appendChild(bookName);
					
					var author = document.createElementNS(config.XHTML_NS, "html:p");
					var text2 = document.createTextNode(bookAuthor);
					author.appendChild(text2);
					details.appendChild(author);
					
					container.appendChild(details);
					
					var btn_container = document.createElementNS(config.XHTML_NS, "html:div");
					btn_container.setAttribute('class', 'thumb-buttons');
					
					if (bookstate == "READYFORDOWNLOAD") {
						var btn1 = document.createElementNS(config.XHTML_NS, "html:button");
						btn1.setAttribute('type', 'button');
						btn1.setAttribute('class', 'thumbnail-download-book');
						btn1.setAttribute('style', 'width: 40px; height: 24px;');
						btn1.setAttribute('onclick', 'AME.downloadPubBook("'+bookId+'");event.stopPropagation();');
						var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
						btnImg.setAttribute('src', 'icons/ico_download_18.png')
						btn1.appendChild(btnImg);
						btn_container.appendChild(btn1);
					} else if (bookstate == "READY") {
						var btn1 = document.createElementNS(config.XHTML_NS, "html:button");
						btn1.setAttribute('type', 'button');
						btn1.setAttribute('class', 'thumbnail-delete-book');
						btn1.setAttribute('style', 'width: 40px; height: 24px;');
						btn1.setAttribute('onclick', 'javascript: AME.deletePubBook("'+bookId+'");event.stopPropagation();');
						var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
						btnImg.setAttribute('src', 'icons/ico_delete-book_18.png')
						btn1.appendChild(btnImg);
						btn_container.appendChild(btn1);
					}
					
					var btn2 = document.createElementNS(config.XHTML_NS, "html:button");
					btn2.setAttribute('type', 'button');
					//btn1.setAttribute('class', 'thumbnail-book-info');
					btn2.setAttribute('style', 'width: 40px; height: 24px;');
					btn2.setAttribute('onclick', 'javascript: AME.showBookInfoDialog("'+bookId+'");event.stopPropagation();');
					var btnImg = document.createElementNS(config.XHTML_NS, 'html:img');
					btnImg.setAttribute('src', 'icons/ico_book-info_18.png')
					btn2.appendChild(btnImg);
					btn_container.appendChild(btn2);
					
					container.appendChild(btn_container);
					
					document.getElementById('show_thumbnail_view').appendChild(container);
				}
			}
		} else if(config.libraryView == 'stack'){
			//	TODO: Implement library view in stack mode.
		} else if(config.libraryView == 'search'){
			//	TODO: Implement library view in stack mode.
			document.getElementById('but_table').click();
		} 
		} catch(e) {
			alert(e);
		}
	}
	
	/**
	 * Sorts the library depending upon the column clicked.
	 * @params column Name of the column to sort against
	 */
	var sort = function(column){
		var self = this;
		
		var tree = document.getElementById("library-azd");
		var order = tree.getAttribute("sortDirection") == "ascending" ? 1 : -1;
		var columnName;
		if (column) {
			columnName = column.id;
			if (tree.getAttribute("sortResource") == columnName) {
				order *= -1;
			}
		}
		else {
			//If column is not provided then sort against created_date.
			columnName = tree.getAttribute("sortResource");
		}
		
		var columnSort = function(a, b){
			if(self.prepareForComparison(a[columnName]) > self.prepareForComparison(b[columnName])) 
	            return 1 * order;
	        if(self.prepareForComparison(a[columnName]) < self.prepareForComparison(b[columnName])) 
	            return -1 * order;
	        //tie breaker: name ascending is the second level sort
	        if(columnName != "title"){
	            if (self.prepareForComparison(a["title"]) > self.prepareForComparison(b["title"])) 
	                return 1;
	            if (self.prepareForComparison(a["title"]) < self.prepareForComparison(b["title"])) 
	                return -1;
	        }
	        return 0;
	    }
	
		config.bookMetadata.sort(columnSort);
		
		tree.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
		tree.setAttribute("sortResource", columnName);
	};
	
	/**
	 * Converts the string to lowercase.
	 * @param o String to convert
	 */
	var prepareForComparison = function(o){
		if(typeof o == "string"){
			return o.toLowerCase();
		}
		return o;
	};
	
	/**
	 * This function returns the bookId from the clicked row.
	 * @returns {string} returns the bookId.
	 */
	var getClickedBook = function(){
		var tree = document.getElementById("library-azd");
    	var cellIndex = 0;
	    var cellText = tree.view.getCellText(tree.currentIndex, tree.columns.getColumnAt(cellIndex));
	    return cellText;
	};
	
	var getClickedBookState = function(){
		var tree = document.getElementById("library-azd");
    	var cellstate = config.bookMetadata[tree.currentIndex]["bookstate"];
		return cellstate;
	};
	
	var getClickedBookIndexState = function(){
		var tree = document.getElementById("library-azd");
    	var indexstatus = INDEXER.existsInIndex(config.bookMetadata[tree.currentIndex]["id"]);
		return indexstatus;
	};
	
	var setClickedBookState = function(state, uid, hash){
		var tree = document.getElementById("library-azd");
    	config.bookMetadata[tree.currentIndex]["bookstate"] = state;
    	config.bookMetadata[tree.currentIndex][uid] = hash;
	};
	
	var getClickedBookMD = function(){
		var tree = document.getElementById("library-azd");
    	var md = config.bookMetadata[tree.currentIndex];
		return md;
	};
	
	var getClickedBookMDByID = function(bid){
		for (var i = 0; i < config.bookMetadata.length; i++) {
			var bookId = config.bookMetadata[i]['id'];
			if (bookId == bid) {
				return config.bookMetadata[i];
			}	
		}
		return {};
	};
	
	/**
	 * Deletes the book from library
	 * @params bookId Id of the book to be deleted
	 */
	var deleteBook = function(bookId){
		var self = this;
		
		var fileObj = getFileObject(AME.getDbFilePath());
		var table_name = 'library_master';
		var condition = new Array(['id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, condition);
		
		table_name = 'shelf_document_join';
		var condition = new Array(['lib_id', bookId, 'INTEGER']);
		deleteRowFromDatabase(fileObj, table_name, condition);
		
		 //DC - FLO UPDATE - 201211 - Delete additional book info
		var table_name = 'book_info';
		var conditions = new Array(['book_id', bookId, 'TEXT']);
		deleteRowFromDatabase(fileObj, table_name, conditions);
		
	};
	
	/**
	 * Displays books only from particular shelf
	 * @params shelfId Id of the shelf
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Short description of the shelf
	 */
	var showShelfBooks = function(shelfId, shelfName, shelfDescription){
		var self = this;
		
		config.library_type = "REGULAR";
		config.shelfId = shelfId;
		
		$("#mi_lib_delete").show();
		$("#mi_lib_openlast").show();
		$("#mi_index").show();
		$("#mi_deindex").show();
		
		//Remove all the books from library which is used to display the library.
		self.clear();
		//Just the following variable will make the library to populate.
		config.bookMetadata = BookMetadata.getShelfBooks(shelfId);
		//Sort the books in the library
		self.sort();
		
		//Set the library view, table|thumbnail
		self.setLibraryView();
		//set the shelf description at the top of library.
		self.showShelfDescription(shelfName, shelfDescription);
	};
	
	var showPubShelfBooks = function(shelfId, shelfName, shelfDescription){
		var self = this;
		
		config.library_type = "ACFS";
		config.shelfId = shelfId;
		
		$("#mi_lib_delete").hide();
		$("#mi_lib_openlast").hide();
		//$("#mi_index").hide();
		//$("#mi_deindex").hide();
		
		Shelf.getPubAccountDetails(shelfId);
		//Remove all the books from library which is used to display the library.
		self.clear();
		//Just the following variable will make the library to populate.
		config.bookMetadata = BookMetadata.getBSShelfBooks(shelfId);
		
		//Sort the books in the library
		self.sort();
		
		//Set the library view, table|thumbnail
		self.setPubLibraryView();
		var shelfdisp = Shelf.getPubShelfDisplayInfo(shelfId);
		//set the shelf description at the top of library.
		self.showPubShelfDescription(shelfName, shelfDescription, shelfdisp);
	};
	
	/**
	 * Sets the shelf name and description at the top of library.
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Description of the shelf.
	 */
	var showShelfDescription = function(shelfName, shelfDescription){
		//Shorten the name and description if length greater than 80 chars.
		shelfDescription = "";
		if (shelfName == "All Books"){
			var allb_label = getPropertiesString("azd_ale_shelf_allBookLabel");
			shelfName = allb_label;
		}
		
		var shelfNameShort = shelfName,
			shelfDescriptionShort = shelfDescription;
		if (shelfName.length > 80) {
			shelfNameShort = shelfName.substr(0, 77) + '...';
		}
		if (shelfDescription.length > 80) {
			shelfDescriptionShort = shelfDescription.substr(0, 77) + '...';
		}
		$('.shelf-source-ape').val("");
		$('.shelf-email-ape').val("");
		$('.shelf-acct-ape').val("");
		$('.pub-shelf').hide();

		$('.shelf-name-ape').val(shelfNameShort.toUpperCase());
		$('.shelf-name-ape').attr('tooltiptext', shelfName.toUpperCase());
		$('.shelf-description-ape').val(shelfDescriptionShort);
		$('.shelf-description-ape').attr('tooltiptext', shelfDescription);
	};
	
	/**
	 * Sets the shelf name, userinfo and description at the top of library.
	 * @params shelfName Name of the shelf
	 * @params shelfDescription Description of the shelf.
	 */
	var showPubShelfDescription = function(shelfName, shelfDescription, extdisp){
		//Shorten the name and description if length greater than 80 chars.
				shelfDescription = "";
		var shelfNameShort = shelfName,
			shelfDescriptionShort = shelfDescription;
		if (shelfName.length > 80) {
			shelfNameShort = shelfName.substr(0, 77) + '...';
		}
		
		$('.pub-shelf').show();

		
		if (shelfDescription.length > 80) {
			shelfDescriptionShort = shelfDescription.substr(0, 77) + '...';
		}
		
		var extdisplaytext = "";
		
		
		if (extdisp["status"]) {
			extdisplaytext = " Source: " + shelfName;
			$('.shelf-source-ape').val(extdisplaytext);
			if (extdisp.hasOwnProperty("email")) {
				var un_label = getPropertiesString("azd_lib_userName");
				extdisplaytext = un_label + ' ' + extdisp["email"];
				$('.shelf-email-ape').val(extdisplaytext);
			}
			if (extdisp.hasOwnProperty("shelf_uid") && extdisp["shelf_uid"]) {
				var ur_label = getPropertiesString("azd_lib_userRef");
				extdisplaytext = ur_label + ' ' + extdisp["shelf_uid"];
				$('.shelf-acct-ape').val(extdisplaytext);
			} else {
				$('.shelf-acct-ape').val("");
			}
		}
		
		
		$('.shelf-name-ape').val(shelfNameShort.toUpperCase());
		$('.shelf-name-ape').attr('tooltiptext', shelfName.toUpperCase());
		$('.shelf-description-ape').val(shelfDescriptionShort);
		$('.shelf-description-ape').attr('tooltiptext', shelfDescription);
	};
	
	var setFilterBy = function() {
		var filter_by = $("#filter_library_by").val();
		if (filter_by) {
			config.filter_on = filter_by;
			self.setLibraryView();
		}
	};
	
	var setFilterText = function(event) {
		var self = this;
		var value = self.prepareForComparison(event.target.value);
		if (value) {
			config.filter_text = value;
			self.setLibraryView();
		} else {
			config.filter_text = "";
			self.setLibraryView();
		}
	};
	
	return {
		config: config,
		treeView: treeView,
		clear: clear,
		init: init,
		sort: sort,
		prepareForComparison: prepareForComparison,
		setLibraryView: setLibraryView,
		getClickedBook: getClickedBook,
		getClickedBookMD: getClickedBookMD,
		getClickedBookMDByID: getClickedBookMDByID,
		deleteBook: deleteBook,
		showShelfBooks: showShelfBooks,
		showPubShelfBooks: showPubShelfBooks,
		getClickedBookState: getClickedBookState,
		setClickedBookState: setClickedBookState,
		setPubLibraryView: setPubLibraryView,
		showShelfDescription: showShelfDescription,
		showPubShelfDescription: showPubShelfDescription,
		setFilterBy: setFilterBy,
		setFilterText: setFilterText,
		getClickedBookIndexState: getClickedBookIndexState,
	};
}();

window.addEventListener("contextmenu", function(e) { 
    var menu = document.getElementById('lib_context_menu');
	var bookstate = Library.getClickedBookState();
	
    if(bookstate == "READYFORDOWNLOAD") {
		$("#mi_lib_delete").hide();
		$("#mi_lib_download").show();
		$("#mi_lib_open").hide();
		$("#mi_index").hide();
	} else if(bookstate == "INACTIVE") {
		$("#mi_lib_delete").hide();
		$("#mi_lib_download").hide();
		$("#mi_lib_open").hide();
		$("#mi_index").hide();
	} else if(bookstate == "EXPIRED") {
		$("#mi_lib_delete").hide();
		$("#mi_lib_download").hide();
		$("#mi_lib_open").hide();
		$("#mi_index").hide();
	} else {
		$("#mi_lib_delete").show();
		$("#mi_lib_open").show();
		$("#mi_lib_download").hide();
		$("#mi_index").show();
	}
	
	if (Library.getClickedBookIndexState() ){
		$("#mi_index").attr("label", "Deindex");
		$("#mi_index").addClass("indexed");
	} else {
		$("#mi_index").removeClass("indexed");
		$("#mi_index").attr("label", "Index");
	}
	//$("#mi_index").show();
	
	
}, false);