/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2012 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2012-04-25
 * 
 * @version: 1-2012-04-25
 * @author Anup Dhabarde
 */

/**
 Function to get values from database.
 @param fileObj			-----	object of .sqlite file	
 @param values			-----	array of values to be retrieved
 @param table_name		-----	table name
 @param condition		-----	2d array of column name and value and column type
  
 @returns statement object
 */

function getValuesFromDatabase(fileObj, values, table_name, condition){
	var valueString = values.join(', ');
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	if(condition != undefined){
		var conditionStatement = 'where '+condition[0][0]+'=?1';
		for(var i=1; i<condition.length; i++){
			conditionStatement += ' and '+condition[i][0]+'=?'+(i+1);
		}
		try {
			var statement = mDBConn.createStatement('SELECT ' + valueString + ' FROM ' + table_name + ' ' + conditionStatement);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
		for(var i=0; i<condition.length; i++){
			switch (condition[i][2]){
				case 'TEXT':
					statement.bindStringParameter(i, condition[i][1]);
					break;
				case 'INTEGER':
					statement.bindInt32Parameter(i, condition[i][1]);
					break;
				case 'DOUBLE':
					statement.bindDoubleParameter(i, condition[i][1]);
					break;
				case 'BLOB':
					statement.bindBlobParameter(i, condition[i][1]);
					break;
				default:
					
			}
		}
	}
	else{
		try {
			var statement = mDBConn.createStatement('SELECT ' + valueString + ' FROM ' + table_name);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
	}
	return statement;
}

/**
 Function to create table in database.
 @param {Object} fileObj
 @param {Object} query
 */

function createTable(fileObj, query){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
}

/**
 Function to delete row from database.
 @param {Object} fileObj
 @param {Object} table_name
 @param {Object} conditions
 */

function deleteRowFromDatabase(fileObj, table_name, conditions){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var conditionString = '';
	
	for(i=0; i<conditions.length; i++){
		if(i>0){
			conditionString += ' AND ';
		}
		conditionString += conditions[i][0];
		conditionString += '=';
		conditionString += '?'+(i+1);
	}
	
	var query =  'DELETE FROM '+table_name+' WHERE '+conditionString;

	var statement = mDBConn.createStatement(query);
	
	for(var i=0; i<conditions.length; i++){
		switch (conditions[i][2]){
			case 'TEXT':
				statement.bindStringParameter(i, conditions[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(i, conditions[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(i, conditions[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(i, conditions[i][1], conditions[i][1].length);
				break;
			default:
		}
	}
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
		dump("\ndeleteRowFromDatabase exception" + e);
	}
	statement.finalize();
	mDBConn.close();
}

function deleteMultiRowFromDatabase(fileObj, table_name, conditions){
	
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var conditionString = '';
	
	for(i=0; i<conditions.length; i++){
		if(i>0){
			conditionString += ', ';
		}
		else {
			conditionString += '';	
		}
		conditionString += '?'+(i+1);
	}
	
	var query =  'DELETE FROM '+table_name+' WHERE uid IN (' + conditionString + ')';
	
	var statement = mDBConn.createStatement(query);
	
	for(var i=0; i<conditions.length; i++){
		switch (conditions[i][2]){
			case 'TEXT':
				statement.bindStringParameter(i, conditions[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(i, conditions[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(i, conditions[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(i, conditions[i][1], conditions[i][1].length);
				break;
			default:
		}
	}
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
		dump("\ndeleteRowFromDatabase exception" + e);
	}
	statement.finalize();
	mDBConn.close();
}


/**
 * Function to delete all rows from table.
 */

function deleteAllRowsFromTable(fileObj, table_name) {
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var query =  'DELETE FROM '+table_name;
	
	var statement = mDBConn.createStatement(query);
	
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
	}
	statement.finalize();
	mDBConn.close();
}

/**
 * 	Function to vacuum the database
 * @param fileObj		-----	object of .sqlite file
 */
function vacuumDatabase(fileObj) {
	//return;
	try{
		var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);
		var mDBConn = storageService.openDatabase(fileObj);
		var query = 'VACUUM';
		var statement = mDBConn.createStatement(query);
		statement.execute();
		statement.finalize();
		mDBConn.close();
	} catch(e){
		dump("\nvacuumDatabase EXCEPTION" + e);
	}
}

function createPubTables(fileObj) {
	try{
		var query = 'CREATE TABLE IF NOT EXISTS shelf_info ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"shelf_id" INTEGER, "shelf_uid" TEXT, "shelf_url" TEXT, "email" TEXT, "shelf_metadata" TEXT);';
		var query1 = 'CREATE TABLE IF NOT EXISTS pub_docs ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"shelf_id" INTEGER, "type" TEXT, "workuid" TEXT, "bookstate" TEXT, "created_date" TEXT, "title" TEXT, "author" TEXT, "publisher_identifier" TEXT, "publish_date" TEXT, "description" TEXT, "location" TEXT, "cover_image" TEXT, "cover_type" TEXT, "version" TEXT, "contributor" TEXT, "format" TEXT, "language" TEXT, "subject" TEXT, "source" TEXT, "relation" TEXT, "coverage" TEXT, "rights" TEXT, "docinfo" TEXT, "dctype" TEXT);';
		var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
		var mDBConn = storageService.openDatabase(fileObj);
		mDBConn.executeSimpleSQL(query);
		mDBConn.executeSimpleSQL(query1);
		
		var query3 = "CREATE TABLE IF NOT EXISTS raf_master ('id' INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,'shelf_id' TEXT ,'agreement_uid' TEXT, 'sd' TEXT, 'ed' TEXT, createdatetime text DEFAULT (strftime('%Y-%m-%d %H:%M:%S:%s','now', 'localtime')),updatedatetime text DEFAULT (strftime('%Y-%m-%d %H:%M:%S:%s','now', 'localtime')));";
		mDBConn.executeSimpleSQL(query3);
		
		
		var query2 = "CREATE TRIGGER IF NOT EXISTS update_raf_md   AFTER UPDATE ON raf_master begin update raf_master set updatedatetime = strftime('%Y-%m-%d %H:%M:%S:%s','now', 'localtime') where id = old.id; end;";
		mDBConn.executeSimpleSQL(query2);
		
		var query4 = "PRAGMA table_info('library_master')";
		var statement = mDBConn.createStatement(query4);
		
		var dctypefound = false;
		while(statement.executeStep()){
			if (statement.row.name == "dctype") {
				dctypefound = true;
			}
		}
		statement.finalize();
		if(!dctypefound) {
			addDCTypeFields(mDBConn, "library_master");
		}
		
		var query5 = "PRAGMA table_info('pub_docs')";
		var statement = mDBConn.createStatement(query5);
		
		dctypefound = false;
		while(statement.executeStep()){
			if (statement.row.name == "dctype") {
				dctypefound = true;
			}
		}
		statement.finalize();
		
		if(!dctypefound) {
			addDCTypeFields(mDBConn, "pub_docs");
		}
		
		mDBConn.close();
	}catch(e){
		dump(e);
		alert(e);
	}
	//update raf_master SET agreement_uid='testfdsfsd8' WHERE agreement_uid='test8';
}

function addDCTypeFields(mDBConn, tablename) {
	var fieldnames = ["dctype", "contributor", "format", "language", "subject", "source", "relation",  "coverage",  "rights"];
	for (var i=0; i<fieldnames.length; i++) {
		var fname = fieldnames[i];
		var query = "ALTER TABLE " + tablename + " ADD COLUMN " + fname +  " TEXT";
		dump("\n------------------" + query);
		mDBConn.executeSimpleSQL(query);
	}
}

function createSITables(fileObj) {
	try{
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var query4 = "CREATE VIRTUAL TABLE IF NOT EXISTS bi USING fts4(shelfid, bookid, title, content);";
	mDBConn.executeSimpleSQL(query4);
	
	var query5 = "CREATE VIRTUAL TABLE IF NOT EXISTS pi USING fts4(shelfid, bookid, title, fn, idref, content);";
	mDBConn.executeSimpleSQL(query5);
	
	var query6 = "CREATE VIRTUAL TABLE IF NOT EXISTS bsbi USING fts4(shelfid, bookid, title, content);";
	mDBConn.executeSimpleSQL(query6);
	
	var query7 = "CREATE VIRTUAL TABLE IF NOT EXISTS bspi USING fts4(shelfid, bookid, title, fn, idref, content);";
	mDBConn.executeSimpleSQL(query7);
	
	mDBConn.close();
	}catch(e){
		alert(e);
	}
}

function createPubDataTables(fileObj) {
	try{
	var query = 'CREATE TABLE IF NOT EXISTS store ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"uid" TEXT, "type" TEXT, "data" TEXT);';
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
	}catch(e){
		alert(e);
	}
}

/**
 Function to insert row into database
 @param fileObj			-----	object of .sqlite file
 @param values			-----	2d array of column name, value and column type
 @param table_name		-----	table name
 */

function insertRowIntoDatabase(fileObj, values, table_name){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	var lastrowid = -1;
	var columnString = '(';
	var valueString = '(';
	
	for(i=0; i<values.length; i++){
		if(i>0){
			columnString += ', ';
			valueString += ', ';
		}
		dump("\n" + values[i][0]);
		columnString += values[i][0];
		valueString += '?'+(i+1);
	}
	columnString += ')';
	valueString += ')';
	
	var query =  'INSERT INTO '+table_name+' '+columnString+' VALUES '+valueString;
	var statement = mDBConn.createStatement(query);
	
	for(var i=0; i<values.length; i++){
		switch (values[i][2]){
			case 'TEXT':
				statement.bindStringParameter(i, values[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(i, values[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(i, values[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(i, values[i][1], values[i][1].length);
				break;
			default:
				
		}
	}
	try{
		statement.execute();
		 //DC - FLO UPDATE - 201211 - get the inserted row id
		try {
			lastrowid = mDBConn.lastInsertRowID;
		} catch(e){
		}
	}
	catch(e){
		lastrowid = -1;
	}
	statement.finalize();
	mDBConn.close();
	 //DC - FLO UPDATE - 201211 - return the last inserted row id
	return lastrowid;
}

/**
 Function to update row of table in database.
 @param {Object} fileObj
 @param {Object} values
 @param {Object} table_name
 @param {Object} conditions
 */

function updateRowIntoDatabase(fileObj, values, table_name, conditions){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	var valueToInsert = '';
	var conditionString = '';
	var totalParameters = 0;
	
	for(i=0; i<values.length; i++){
		totalParameters++;
		if(i>0){
			valueToInsert += ', ';
		}
		valueToInsert += values[i][0];
		valueToInsert += '=';
		valueToInsert += '?'+totalParameters;
	}
	
	for(i=0; i<conditions.length; i++){
		totalParameters++;
		if(i>0){
			conditionString += 'and ';
		}
		conditionString += conditions[i][0];
		conditionString += '=';
		conditionString += '?'+totalParameters;
	}
	
	var query =  'UPDATE '+table_name+' SET '+valueToInsert+' WHERE '+conditionString;
	
	var statement = mDBConn.createStatement(query);
	
	totalParameters = -1;
	
	//	binding values.
	for(var i=0; i<values.length; i++){
		totalParameters++;
		switch (values[i][2]){
			case 'TEXT':
				statement.bindStringParameter(totalParameters, values[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(totalParameters, values[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(totalParameters, values[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(totalParameters, values[i][1], values[i][1].length);
				break;
			default:
		}
	}
	
	//	binding conditions
	for(var i=0; i<conditions.length; i++){
		totalParameters++;
		switch (conditions[i][2]){
			case 'TEXT':
				statement.bindStringParameter(totalParameters, conditions[i][1]);
				break;
			case 'INTEGER':
				statement.bindInt32Parameter(totalParameters, conditions[i][1]);
				break;
			case 'DOUBLE':
				statement.bindDoubleParameter(totalParameters, conditions[i][1]);
				break;
			case 'BLOB':
				statement.bindBlobParameter(totalParameters, conditions[i][1], conditions[i][1].length);
				break;
			default:
		}
	}
	try{
		statement.execute();
	}
	catch(e){
		alert(e);
	}
	statement.finalize();
	mDBConn.close();
}

function getValuesFromDatabaseJoin(fileObj, queryString, condition){
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	
	if(condition != undefined){
		try {
			var statement = mDBConn.createStatement(queryString);
		}
		catch(e){
			dump(e+' '+e.message+'\n');
			return false;
		}
		for(var i=0; i<condition.length; i++){
			switch (condition[i][1]){
				case 'TEXT':
					statement.bindStringParameter(i, condition[i][0]);
					break;
				case 'INTEGER':
					statement.bindInt32Parameter(i, condition[i][0]);
					break;
				case 'DOUBLE':
					statement.bindDoubleParameter(i, condition[i][0]);
					break;
				case 'BLOB':
					statement.bindBlobParameter(i, condition[i][0]);
					break;
				default:
					
			}
		}
	}
	else{
		try {
			var statement = mDBConn.createStatement(queryString);
		}
		catch(e){
			dump(e+' '+e.message);
			return false;
		}
	}
	
	return statement;
}

function createBookInfoTable(fileObj, tablename) {
	var query = 'CREATE TABLE IF NOT EXISTS ' + tablename + ' ("id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL ,"book_id" TEXT, "bookinfo" TEXT)';
	var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
	var mDBConn = storageService.openDatabase(fileObj);
	mDBConn.executeSimpleSQL(query);
	mDBConn.close();
}

