/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2014 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2014-05-19
 * 
 * @version: 1-2014-05-19
 * @author Deepak Chandran
 */

var MOCK_RAF = {};
var MOCK_RAF_TOKEN = {};
 
var RAF = function(){
	var getTokenInfo = function(ag_uid) {
		try {
			var fileObj = getFileObject(AME.getDbFilePath());		
			var table_name = 'raf_master';
			var res = {};
			var values = ['sd', 'ed', 'updatedatetime'];
			var conditions = new Array(['agreement_uid', ag_uid, 'TEXT']);
			var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			var currDate_s = "";
			while (statement.executeStep()){
				res["sd"] = statement.row.sd;
				res["ed"] = statement.row.ed;
				res["cd"] = statement.row.updatedatetime;
			}
			statement.finalize();
			
			return res;
		} catch(e) {
			alert("getTokenInfo EXCEPTION " + e);
			return {};
		}
	};
	
	var getTokenSeal = function(ag_uid, shelfid) {
		try {
			var enc_full = "";
			var shelfinfo = Shelf.getPubAcctMd(shelfid);
			if (!shelfinfo) {
				return "";
			}
			
			if (shelfinfo.hasOwnProperty(ag_uid+"tok")) {
				enc_full = shelfinfo[ag_uid+"tok"];
			}
			
			return enc_full;
		} catch(e) {
			alert("getTokenSeal EXCEPTION" + e);
			return "";
		}
	};

	var saveConfirmationToken = function(ag_uid, enc_full, shelfid) {
		try {
			var shelfinfo = Shelf.getPubAcctMd(shelfid);
			if (!shelfinfo) {
				alert("shelfinfo not found");
				return false;
			}
			shelfinfo[ag_uid+"tok"] = enc_full;
			
			var fileObj = getFileObject(AME.getDbFilePath());
			var table_name = "shelf_info";
			var values = new Array(['shelf_metadata', JSON.stringify(shelfinfo), 'TEXT']);
			var conditions = new Array(['shelf_id', shelfid, 'TEXT']);
			updateRowIntoDatabase(fileObj, values, table_name, conditions);
		} catch(e) {
			alert("saveConfirmationToken EXCEPTION" + e);
		}
	};
	
	var saveRAF = function(ag_uid, sd, ed, shelfid){
		try {
			var currDate = new XDate();
			var currDate_s = currDate.toString();
			var fileObj = getFileObject(AME.getDbFilePath());		
			var table_name = 'raf_master';
			var values = new Array(
								['shelf_id', shelfid, 'TEXT'],
								['agreement_uid', ag_uid, 'TEXT'],
								['sd', sd, 'TEXT'],
								['ed', ed, 'TEXT']
								);
			var lastid = insertRowIntoDatabase(fileObj, values, table_name);	
			
			values = ['updatedatetime'];
			var conditions = new Array(['id', lastid, 'INTEGER']);
			var statement = getValuesFromDatabase(fileObj, values, table_name, conditions);
			currDate_s = "";
			while (statement.executeStep()){
				currDate_s = statement.row.updatedatetime;
			}
			statement.finalize();
			
			return currDate_s;
		} catch(e) {
			alert("saveRAF EXCEPTION" + e);
		}
	};
	
	var createRAFInfo = function(devtok, ag_uid, sd, ed, shelfid){	
		try {
			var fullkey = devtok + ag_uid;
			var enc_sd = CryptoJS.AES.encrypt(sd, fullkey);
			var enc_ed = CryptoJS.AES.encrypt(ed, fullkey);
			var md = saveRAF(ag_uid, enc_sd.toString(), enc_ed.toString(), shelfid);
			var enc_source = ag_uid + enc_sd.toString() + enc_ed.toString() + md;
			var enc_full = CryptoJS.AES.encrypt(enc_source, fullkey);
			saveConfirmationToken(ag_uid, enc_full.toString(), shelfid);
		} catch(e) {
			alert("createRAFInfo EXCEPTION" + e);
		}
	};
	
	var verifyAgreement = function(devtok, ag_uid, shelfid) {
		try {
			var res = {"status": false, "reason": ""};
			var fullkey = devtok + ag_uid;
			var tok = getTokenInfo(ag_uid);
			var enc_source = ag_uid + tok["sd"] + tok["ed"] + tok["cd"];
			var enc_full = CryptoJS.AES.encrypt(enc_source, fullkey);
			var token_seal = getTokenSeal(ag_uid, shelfid);
			token_seal = CryptoJS.AES.decrypt(token_seal, fullkey);
			
			if (enc_source != token_seal.toString(CryptoJS.enc.Utf8)) {
				//Token data was modified afterwards
				res["reason"] = "Token modified error";
				return res;
			}
			
			var dec_sd = CryptoJS.AES.decrypt(tok["sd"], fullkey);
			var dec_ed = CryptoJS.AES.decrypt(tok["ed"], fullkey);
			var start_date = new XDate(dec_sd.toString(CryptoJS.enc.Utf8));
			var end_date = new XDate(dec_ed.toString(CryptoJS.enc.Utf8));
			var currDate = new XDate();
			
			var diff_sd = currDate.diffDays(start_date);
			var diff_ed = currDate.diffDays(end_date);
			
			if (diff_sd > 0) {
				//Current date > Subscription startdate
				res["reason"] = "Waiting";
				return res;
			}
			if (diff_ed <= 0) {	
				//Subscription enddate < Current date 
				res["reason"] = "Expired";
				return res;
			}
			res["status"] = true;
			res["reason"] = "";
			return res;
		} catch(e) {
			res["status"] = false;
			res["reason"] = "verifyAgreement EXCEPTION" + e;
			return res;
		}
	};
	
	var getAgreementDates = function(devtok, ag_uid) {
		try {
			var res = {"status": false, "reason": ""};
			var fullkey = devtok + ag_uid;
			var tok = getTokenInfo(ag_uid);
			
			var dec_sd = CryptoJS.AES.decrypt(tok["sd"], fullkey);
			var dec_ed = CryptoJS.AES.decrypt(tok["ed"], fullkey);
			
			res["status"] = true;
			res["reason"] = "";toLocaleString
			var start_date = new XDate(dec_sd.toString(CryptoJS.enc.Utf8));
			res["sd"] = start_date.toString("dd MMM yyyy");
			var end_date = new XDate(dec_ed.toString(CryptoJS.enc.Utf8));
			res["ed"] = end_date.toString("dd MMM yyyy");
			return res;
		} catch(e) {
			res["status"] = false;
			res["reason"] = "verifyAgreement EXCEPTION" + e;
			res["sd"] = "";
			res["ed"] = "";
			return res;
		}
	};
	
	return {
		createRAFInfo: createRAFInfo,
		getAgreementDates: getAgreementDates,
		verifyAgreement: verifyAgreement
	}
}();

var RAF_TEST = function() {
	
	var run = function() {
		var dev_tok = "b108efaf65532bc0a6ed4fb0c873ff83";
		var ag_uid = "AAAAAAA";
		var ag_uid = "AAAAAAA";
		var start_date = "2014-05-14 00:30:00";
		var end_date = "2014-05-21 20:30:00";
		RAF.createRAFInfo(dev_tok, ag_uid, start_date, end_date);
		var res = RAF.verifyAgreement(dev_tok, ag_uid);
		console.log("Regular test", res);
		setTimeout( function(){
				updateModifiedTime(ag_uid);
				var res = RAF.verifyAgreement(dev_tok, ag_uid);
				console.log("Test after updating the modified date time", res);
		}, 2000);
		
	}
	
	var updateModifiedTime = function(ag_uid){
		if (MOCK_RAF.hasOwnProperty(ag_uid)) {
			var currDate = new XDate();
			var currDate_s = currDate.toString();
			MOCK_RAF[ag_uid]["cd"] = currDate_s;
		}
	}
	
	return {
		run: run
	}
}();
