/**
 * AZARDI READER AND AZARDI INTERACTIVE ENGINE EXTENSIONS
 * Copyright 2007-2014 Infogrid Pacific Pte. Ltd. All rights reserved
 * Modified Date: 2014-05-27
 * 
 * @version: 1-2014-27-19
 * @author Deepak Chandran
 */

 
var INDEXER = function(){
	var config = {
		indexstate: {},
		hilite_ids: []
	};
	
	var deleteBookFromIndex = function(identifier, shelfId) {
		try {
			if(Shelf.config.currentShelfType == "acfs"){
				deleteBSBookFromIndex(identifier, shelfId);
				return false;
			}
			
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = 'bi';
			var conditions = new Array(['bookid', identifier, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, conditions);
			
			table_name = 'pi';
			var conditions = new Array(['bookid', identifier, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, conditions);
		} catch(e) {
			alert(e);
		}
	};
	
	var deleteBSBookFromIndex = function(identifier, shelfId) {
		try {
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = 'bsbi';
			var conditions = new Array(['bookid', identifier, 'TEXT'], ['shelfid', shelfId, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, conditions);
			
			table_name = 'bspi';
			var conditions = new Array(['bookid', identifier, 'TEXT'], ['shelfid', shelfId, 'TEXT']);
			deleteRowFromDatabase(fileObj, table_name, conditions);
		} catch(e) {
			alert(e);
		}
	};
	
	var extractSectionTitle = function(domTree) {
		var node = domTree.getElementsByTagName("h1").item(0);
		if (node) {
			return node.innerHTML;
		}
		
		var node = domTree.getElementsByTagName("h2").item(0);
		if (node) {
			return node.innerHTML;
		}
		
		var node = domTree.getElementsByTagName("h3").item(0);
		if (node) {
			return node.innerHTML;
		}
		
		var node = domTree.getElementsByTagName("h4").item(0);
		if (node) {
			return node.innerHTML;
		}
		
		return "";
	};
	
	var index = function(bookLocation, bookTitle, bookAuthor, identifier, shelfId, from_ctx_menu) {
		try {
			Epub.unload();
			var epubFile = getFileObjectFromPath(bookLocation);
			if(!epubFile.exists() || !epubFile.isFile()) {
				var epubnotfound = getPropertiesString("azd_epub_notFound");
				if (!epubnotfound) {
					epubnotfound = "Epub not found."
				}
				if (!AME.config.is_mac) {
					alert(bookTitle + " - " + epubnotfound);
				} else if (AME.config.is_mac && from_ctx_menu) {
					alert(bookTitle + " - " + epubnotfound);
				}				
				return false;
				throw new Error(epubnotfound);
			}
			
			if (shelfId) {
				shelfId = "MASTER";
			}
			
			try{
				unzipFile(epubFile.path, false);
			}
			catch(e){
				//	Throw error if archive file is corrupted.
				alert(bookTitle + ' - An error occurred while trying to load the book.');
				throw new Error(bookTitle + ' - An error occurred while trying to load the book.');
				return false;
			}
			Epub.getExtractLocation();
			Epub.getExtractedFolderName(epubFile.leafName);
			
			var containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder, 'META-INF', 'container.xml']);
			var containerXmlContent = readFileUTF(containerXmlPath);
			var opfFilePath = Epub.getOpfFilePath(containerXmlContent);
			var opfFileContent = readFileUTF(opfFilePath);
			Epub.config.spineList.splice(0, Epub.config.spineList.length);
			//Get the spine of the epub
			Epub.getSpine(opfFileContent);
			Epub.loadMetaProperties(opfFileContent);
			if(Epub.config.version == 3){
				Epub.getManifestEpub3(opfFileContent);
			} else {
				Epub.getManifestEpub2(opfFileContent);
			}
			
			var fullbookcontent = "";
			var leaflookup = {};
			var tot_indexed = 0;
			for(var i = 0; i < Epub.config.bookData.length; i++){
				if(Epub.config.bookData[i]['mediaType'] == 'text/html' || Epub.config.bookData[i]['mediaType'] == 'application/xhtml+xml'){
					try {
						var idref = JSON.stringify(Epub.config.bookData[i]["id"]);
						var inspine = Epub.getChapterIdFromSpineId(idref.replace(/\"/g, ""));
						if (inspine == false ){
							if (inspine != 0) {
								//we only want to index linear yes items from the index
								continue;
							}
						}
						
						var htmlPath = Epub.getEntityAbsolutePath(opfFilePath, Epub.config.bookData[i]['href']);
						var htmlObj = getFileObjectFromPath(htmlPath);
						var filename = htmlObj.leafName.split('.');
						var name = filename[0];
						var extension = filename[1];
						var htmlXmlContent = readFileUTF(htmlPath);
						var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
									.createInstance(Components.interfaces.nsIDOMParser);
						var domTree = parser.parseFromString(htmlXmlContent, "text/html");
						var spineNode = domTree.getElementsByTagName("body").item(0);
						if (spineNode) {
							htmlXmlContent = spineNode.innerHTML;
						}
						var sectitle = extractSectionTitle(domTree);
						var strippedstring = htmlXmlContent.replace(/(<([^>]+)>)/ig,"");
						fullbookcontent = fullbookcontent + strippedstring;
						leaflookup[filename] = {"c": strippedstring, "idref": idref, "sectitle": sectitle};
						tot_indexed = tot_indexed + 1;
					} catch(e) {
						alert(e);
					}
				}
			}
			
			deleteBookFromIndex(identifier, shelfId);
			
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "pi";
			var i = 0;
			
			for (fn in leaflookup) {
				var percentComplete = (i/tot_indexed)*100;
				//dump("\n" + percentComplete + "\t" + i + "\t" + tot_indexed + "\t" + i/tot_indexed);
				//$('#download-progressbar').attr('value', percentComplete);
				
				var idref = leaflookup[fn]["idref"];
				var c = leaflookup[fn]["c"];
				var st = leaflookup[fn]["sectitle"];
				var values = new Array(['shelfid', shelfId, 'TEXT'],
					['bookid', identifier, 'TEXT'],
					['title', st, 'TEXT'],
					['fn', fn, 'TEXT'],
					['idref', idref, 'TEXT'],
					['content', c, 'TEXT']
				);
				insertRowIntoDatabase(fileObj, values, table_name);
				i = i + 1;
			}
			
			var table_name = "bi";
			var values = new Array(['shelfid', shelfId, 'TEXT'],
				['bookid', identifier, 'TEXT'],
				['title', bookTitle, 'TEXT'],
				['content', fullbookcontent, 'TEXT']
			);
			insertRowIntoDatabase(fileObj, values, table_name);
			return true;
		} catch(e) {
			if (!AME.config.is_mac) {
				alert(e);
			}			
			return false;
		}
	};
	
	var getSectionInfo = function(id, manifest) {
		for(i = 0; i < manifest.length; i++){
			if (manifest[i]["id"] == id) {
				return manifest[i];
			}
		}
		return {};
	}
	
	var indexBSB = function(bookLocation, bookTitle, bookAuthor, identifier, shelfId, from_ctx_menu, opfData) {
		try {
			if (!opfData) return false;
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "bspi";
			var fullbookcontent = "";
			deleteBSBookFromIndex(identifier, shelfId);
			for(k = 0; k < opfData.spine.length; k++){
				try {
					var idRef = opfData.spine[k];
					dump("\n====================" + idRef + "\t" + identifier);
					
					var secinfo = getSectionInfo(idRef, opfData.manifest);
					if (secinfo['mediaType'] == 'text/html' || secinfo['mediaType'] == 'application/xhtml+xml') {
						var hash = secinfo["hash"];
					
						var sec = BookMetadata.getBSBookAdditionalMetadata(hash);
						var htmlXmlContent = sec["data"];
						var parser = Components.classes["@mozilla.org/xmlextras/domparser;1"]
									.createInstance(Components.interfaces.nsIDOMParser);
						var domTree = parser.parseFromString(htmlXmlContent, "text/html");
						var spineNode = domTree.getElementsByTagName("body").item(0);
						if (spineNode) {
							htmlXmlContent = spineNode.innerHTML;
						}
						var sectitle = extractSectionTitle(domTree);
						var strippedstring = htmlXmlContent.replace(/(<([^>]+)>)/ig,"");
						fullbookcontent = fullbookcontent + strippedstring;
						
						var values = new Array(['shelfid', shelfId, 'TEXT'],
							['bookid', identifier, 'TEXT'],
							['title', sectitle, 'TEXT'],
							['fn', hash, 'TEXT'],
							['idref', idRef, 'TEXT'],
							['content', strippedstring, 'TEXT']
						);
						insertRowIntoDatabase(fileObj, values, table_name);	
					}
				 }catch(e){
					alert(e);
				}
			}
			
			var table_name = "bsbi";
			var values = new Array(['shelfid', shelfId, 'TEXT'],
				['bookid', identifier, 'TEXT'],
				['title', bookTitle, 'TEXT'],
				['content', fullbookcontent, 'TEXT']
			);
			insertRowIntoDatabase(fileObj, values, table_name);
			
			return true;
		} catch(e){
			//alert(e);
			return false;
		}
	};
	
	var indexE0 = function(bookLocation, bookTitle, bookAuthor, identifier, shelfId) {
		Epub.unload();
		var epubFile = getFileObjectFromPath(bookLocation);
		if(!epubFile.exists() || !epubFile.isFile()) {
			var epubnotfound = getPropertiesString("azd_epub_notFound");
			if (!epubnotfound) {
				epubnotfound = "Epub not found."
			}
			throw new Error(epubnotfound);
		}
		
		if (shelfId) {
			shelfId = "MASTER";
		}
		
		try{
			unzipFile(epubFile.path, false);
		}
		catch(e){
			//	Throw error if archive file is corrupted.
			throw new Error('An error occurred while trying to load the book.');
		}
		Epub.getExtractLocation();
		Epub.getExtractedFolderName(epubFile.leafName);
		
		var containerXmlPath = getAppendFilePath(Epub.config.extractLocation, [Epub.config.extractedFolder, 'META-INF', 'container.xml']);
		
		var containerXmlContent = readFileUTF(containerXmlPath);
		var opfFilePath = Epub.getOpfFilePath(containerXmlContent);
		var opfFileContent = readFileUTF(opfFilePath);
		Epub.config.spineList.splice(0, Epub.config.spineList.length);
		//Get the spine of the epub
		Epub.getSpine(opfFileContent);
		Epub.loadMetaProperties(opfFileContent);
		if(Epub.config.version == 3){
			Epub.getManifestEpub3(opfFileContent);
		} else {
			Epub.getManifestEpub2(opfFileContent);
		}
		
		fullbookcontent = "";
		var leaflookup = {};
		for(var i = 0; i < Epub.config.bookData.length; i++){
			if(Epub.config.bookData[i]['mediaType'] == 'text/html' || Epub.config.bookData[i]['mediaType'] == 'application/xhtml+xml'){
				try {
					var idref = JSON.stringify(Epub.config.bookData[i]["id"]);
					if (Epub.getChapterIdFromSpineId(idref.replace(/\"/g, "")) < 0){
						//we only want to index linear yes items from the index
						continue;
					}
					var htmlPath = Epub.getEntityAbsolutePath(opfFilePath, Epub.config.bookData[i]['href']);
					var htmlObj = getFileObjectFromPath(htmlPath);
					var filename = htmlObj.leafName.split('.');
					var name = filename[0];
					var extension = filename[1];
					var htmlXmlContent = readFileUTF(htmlPath);
					var strippedstring = htmlXmlContent.replace(/(<([^>]+)>)/ig,"");
					fullbookcontent = fullbookcontent + strippedstring;
					leaflookup[filename] = {"c": strippedstring, "idref": idref};
				} catch(e) {
					alert("index exception" + e);
				}
			}
		}
		
		deleteBookFromIndex(identifier, shelfId);
		
		var fileObj = getFileObject(AME.config.dbFilePath);
		var table_name = "pi";
		for (fn in leaflookup) {
			var idref = leaflookup[fn]["idref"];
			var c = leaflookup[fn]["c"];
			var values = new Array(['shelfid', shelfId, 'TEXT'],
				['bookid', identifier, 'TEXT'],
				['title', bookTitle, 'TEXT'],
				['fn', fn, 'TEXT'],
				['idref', idref, 'TEXT'],
				['content', c, 'TEXT']
			);
			insertRowIntoDatabase(fileObj, values, table_name);
		}
		
		var table_name = "bi";
		var values = new Array(['shelfid', shelfId, 'TEXT'],
			['bookid', identifier, 'TEXT'],
			['title', bookTitle, 'TEXT'],
			['content', fullbookcontent, 'TEXT']
		);
		insertRowIntoDatabase(fileObj, values, table_name);
	};


	var existsInIndex = function(bid) {
		if(Shelf.config.currentShelfType == "acfs"){
			bid = "acfs-" + bid;
			if (config.indexstate && config.indexstate.hasOwnProperty(bid)){
				return true;
			}
			return false;
		}

		if (config.indexstate && config.indexstate.hasOwnProperty(bid)){
			return true;
		}
		return false;
	};
	
	var getAllIndexedBooks = function() {
		try {
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			if (!fileObj.exists()) {
				return;
			}
			var values = ['bookid', 'title'];
			var tableName = 'bi';
			var statement = getValuesFromDatabase(fileObj, values, tableName);
			var res = {};
			while(statement.executeStep()){
				bookid = statement.row.bookid;
				title = statement.row.title;
				if (bookid) res[bookid] = title
			}
			
			var values = ['bookid', 'title'];
			var tableName = 'bsbi';
			var statement = getValuesFromDatabase(fileObj, values, tableName);
			while(statement.executeStep()){
				bookid = statement.row.bookid;
				bookid = "acfs-" + bookid;
				title = statement.row.title;
				if (bookid) res[bookid] = title
			}
			
			statement.finalize();
			config.indexstate = res;
			return res;
		} catch(e) {
			return {};
		}
	};
	
	return {
		config: config,
		index: index,
		indexE0: indexE0,
		indexBSB: indexBSB,
		existsInIndex: existsInIndex,
		deleteBookFromIndex: deleteBookFromIndex,
		getAllIndexedBooks: getAllIndexedBooks
	}
}();

IGPS = function() {
	var config = {
		items_per_page: 10,
		search_term: "",
		shelfid: "",
		currpage: 1,
		results: [],
		curr_book_title: "",
		clickedItem: "",
		clickedIndex: "",
		currBookID: "",
		hilite_ids: [],
		current_hl_index: 0
	};
	
	/*var preProcessSearchTerm = function(searchTerm) {
		var s = searchTerm;
		var sl, res=[];
		s = s.replace(/AND/ig, "");
		s = s.replace(/OR/ig, "");
		s = s.replace(/NOT/ig, "");
		s = s.replace(/\"/g, "");
		s = s.replace(/\* /g, "");
		s = s.toLowerCase();
		sl = s.split(" ");
		for (var i=0; i<sl.length; i++) {
			if(sl[i]) {
				res.push(sl[i]);
			}
		}
		return res;
	}*/
	
	var preProcessSearchTerm = function(searchterm) {
		var preProcessSearchTerm = function(searchTerm) {
			var s = searchTerm;
			s = $.trim(s);
			s = s.toLowerCase();
			if (s == "and" || s == "or" || s == "not") {
				s = "";
			}
			return s;
		}
		
		var tokenize = function(indata) {
			var res = [];
			var s = "";
			var in_quotes = false;
			for (var i=0; i < indata.length; i++) {
				if (indata[i] == "\"" && !in_quotes) {
					s = preProcessSearchTerm(s);
					if(s){
						res.push(s);
					}
					s="";
					in_quotes = true;
				} else if (indata[i] == "\"" && in_quotes) {
					if(s)res.push(s);
					s="";
					in_quotes = false;
				} else if (indata[i] == " " && !in_quotes) {
					s = preProcessSearchTerm(s);
					if(s)res.push(s);
					s="";
				} else {
					s = s + indata[i];
				}
			}
			
			s = preProcessSearchTerm(s);
			if(s) {
				res.push(s);
			}
			return res;
		}
		
		var processWildcard = function(in_tok) {
			var res = "";
			
			if (in_tok.indexOf("*") < 0) {
				//in_tok =  "\\b" + in_tok + "\\b";
				return in_tok;
			}
			
			var in_tok_list = in_tok.split(" ");
			
			for (var i=0; i < in_tok_list.length; i++) {
				var token = in_tok_list[i];
				if (token[0] == "*"){
					token = token.replace("*", "\\b\\w+");
					token = token + "\\b";
				}
				if (token[token.length-1] == "*"){
					token = token.substring(0, token.length-1) +  "\\w+\\b";
					token = "\\b" + token;
				} else {
					token = token.replace(/\*/ig, "\\w+");
					//token =  "\\b" + token + "\\b";
				}
				
				res = res + token;
			}
			
			return res;
		}
		
		var p_st = searchterm ; //preProcessSearchTerm(searchterm);
		var token_list = tokenize(p_st);
		token_list = processWildcard(token_list);
		for (var i=0; i < token_list.length; i++) {
			var mod_tok = token_list[i];
			mod_tok = processWildcard(mod_tok);
			token_list[i] = mod_tok;
		}
		
		return token_list;
	}
	
	var getClickedItemId = function() {
		if(Shelf.config.currentShelfType == "acfs"){
			return IGPBSS.getClickedItemId();
		}
		
		return config.clickedItem;
	}
	
	var getSearchTerm = function() {
		if(Shelf.config.currentShelfType == "acfs"){
			return IGPBSS.getSearchTerm();
		}
		
		if (!config.search_term)return "";
		var terms = preProcessSearchTerm(config.search_term);
		return terms;
	}
	
	var handleSectionClick = function(bookid, idref, res_index) {
		var openid = bookid;
		/*for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if(Library.config.bookMetadata[i]['id'] === bookid){
				openid = Library.config.bookMetadata[i]['id'];
				break;
			}
		}*/	
		if (openid) {
			config.clickedItem = idref;
			config.clickedIndex = parseInt(res_index);
			config.currBookID = openid;
			AME.openBook(openid, "search");
		}
	};
	
	var displayBookDetailResults = function (results, init_nav) {
		try {
			var final_res = "";
			var bookstitle = "";
			var s_index =  (config.currpage - 1) * config.items_per_page;
			var e_index =  s_index + config.items_per_page;
			if (e_index > results.length) e_index = results.length;
			bookstitle = config.curr_book_title;
			
			var summarytpl = '<html:p class="search_summary"><html:span class="result-booktitle">' + bookstitle + '</html:span> - Found <html:b>"' + config.search_term + '"</html:b> in <html:b>' + results.length;
			
			if (results.length > 1) {
				summarytpl = summarytpl + '</html:b> sections. </html:p>';
			} else {
				summarytpl = summarytpl + '</html:b> section. </html:p>';
			}
			
			for (var i= s_index; i< e_index; i++) {
				row = results[i];
				var booktpl = '<html:li class="book-title" id="$bookid$" onclick="javascript: IGPS.handleSectionClick(\'$bookid$\', \'$idref$\', \'$index$\')"><html:b>$title$</html:b> <html:span class="result-snippet">$br_snip$</html:span><html:span class="result-info">$brc$</html:span></html:li>'
				var bookid = row['bookid'];
				var sec_title = row['title'];
				var snippet = booktpl;
				var pagecount = row['totalhits'];	
				var b_snippet = row['snippet(pi)'];
				var idref = row['idref'].replace(/\"/ig, "");
				idref = escapeTerms(idref);
				
				b_snippet = b_snippet.replace(/<b>/ig, "###b###");
				b_snippet = b_snippet.replace(/<\/b>/ig, "###/b###");
				b_snippet = b_snippet.replace(/(<([^>]+)>)/ig,"");
				b_snippet = b_snippet.replace(/###b###/ig, "<b>");
				b_snippet = b_snippet.replace(/###\/b###/ig, "<\/b>");
				
				var snippet = booktpl;
				snippet = snippet.replace(/\$bookid\$/ig, bookid);
				var rep_title = sec_title;
				if (rep_title) rep_title = sec_title + ": ";
				snippet = snippet.replace("$title$", rep_title);
				snippet = snippet.replace("$idref$", idref);
				snippet = snippet.replace("$brc$", pagecount);
				snippet = snippet.replace("$br_snip$", b_snippet);
				snippet = snippet.replace("$index$", i);
				//if (config.currpage == 3)alert(snippet);
				final_res = final_res + snippet;
			}
			
			if (final_res) { 
				final_res = final_res.replace(/&#([^\s]*);/g, function(match, match2) {return String.fromCharCode(Number(match2));})
				final_res = final_res.replace(/&nbsp;/ig, " ");
				final_res = final_res.replace(/&/ig, " ");
				final_res = final_res.replace(/<b>/ig, "<html:b>");
				final_res = final_res.replace(/<\/b>/ig, "</html:b>");
				final_res = final_res.replace(/<br>/ig, "");
				final_res = summarytpl + "<html:ol>" + final_res + "</html:ol>";
				
			} else {
				final_res = "<html:p class='search_summary'>No results found.</html:p>";
			}
			
			document.getElementById('search_results').innerHTML = final_res;
			
			if (results.length > config.items_per_page && init_nav) {
				$("#sr-pagination").pagination({
					items: results.length,
					itemsOnPage: config.items_per_page,
					cssStyle: 'compact-theme',
					onPageClick: function(pageNumber, event) {
						config.currpage = pageNumber;
						displayBookDetailResults(config.results, false);
					}
				});
				$("#sr-pagination").show();
			} else if(init_nav) {
				$("#sr-pagination").hide();
			}
		} catch(e) {
			
		}
	}
	
	var searchByBookSections = function(bookid, booktitle) {
		try {
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "pi";
			var sql =  'SELECT  shelfid, bookid, title, idref, snippet(pi), hex(matchinfo(pi,"pcx")) FROM pi  ';
			sql = sql +  "  WHERE content MATCH \'" + config.search_term + '\' AND bookid=\''+bookid+ '\'';
			
			//Now open the sqlite database as 
			var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
			var mDBConn = storageService.openDatabase(fileObj);
			var statement = mDBConn.createStatement(sql);
			var results1 = [];
			
			statement.execute();
			
			while (statement.executeStep()) {
				var row = [];				
				for (var i=0, k = statement.columnCount;i<k;i++){
					if (statement.getColumnName(i) === 'hex(matchinfo(pi,"pcx"))') {
						var value = statement.getUTF8String(i);
						var totalhits = convertMatchInfo(value);
						row["totalhits"] = totalhits; 
						row[statement.getColumnName(i)] = value; 
					} else {
						row[statement.getColumnName(i)] = statement.getUTF8String(i); 
					}
				}
				results1.push(row);
			}
		} catch(e) {
			alert(e);
		}
		
		results1.sort(self.compare);
		
		config.results = results1;
		config.currpage = 1;
		config.curr_book_title = booktitle;
		
		displayBookDetailResults(results1, true);
		statement.finalize();
		mDBConn.close();
		return results1;
		
	};
	
	var convertMatchInfo = function(hex_mi) {
		try {
			var res = [];
			var strlen = hex_mi.length;
			var iter_cnt = strlen/8;
			for (var i=0;i<iter_cnt;i++){
				var si = i*8;
				var ei = si + 8;
				var val = hex_mi.substr(si, 8);
				val = val.split("").reverse().join("");
				var_h = parseInt(val, 32);
				var_d = Math.round(var_h/32);
				res.push(var_d);
			}
			
			var phrasecnt = res[0];
			var colcnt = res[1];
			
			var totalhits = 0;
			
			for (var i=2;i<res.length;i=i+3){
				var index = i;
				var hits = res[index];
				totalhits = totalhits + hits;
			}
			
			return totalhits;
		}catch(e){
			alert(e);
			return 0;
		}
	}
	
	compare = function (a,b) {
	  if (a.totalhits > b.totalhits)
		 return -1;
	  if (a.totalhits < b.totalhits)
		return 1;
	  return 0;
	}
	
	var searchByBooks = function(searchterm, shelfid) {
		try {
			if(Shelf.config.currentShelfType == "acfs"){
				IGPBSS.searchByBooks(searchterm, shelfid);
				return false;
			}
			
			var book_metadata_lookup = {};
			searchterm = searchterm.replace(/'/g,"'\'");
			config.search_term = searchterm;
			config.shelfid = shelfid;
			config.currpage = 1;
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "bi";
			var sql =  'SELECT  shelfid, bookid, title, snippet(bi), hex(matchinfo(bi,"pcx")) FROM bi  ';
			sql = sql +  "  WHERE content MATCH \'" + searchterm + '\'';
			
			//Now open the sqlite database as 
			var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
			var mDBConn = storageService.openDatabase(fileObj);
			var statement = mDBConn.createStatement(sql);
			var results1 = [];
			var results2 = [];
			
			statement.execute();
			
			while (statement.executeStep()) {
				var row = [];				
				for (var i=0, k = statement.columnCount;i<k;i++){
					if (statement.getColumnName(i) === 'hex(matchinfo(bi,"pcx"))') {
						var value = statement.getUTF8String(i);
						var totalhits = convertMatchInfo(value);
						row["totalhits"] = totalhits; 
						row[statement.getColumnName(i)] = value; 
					} else {
						row[statement.getColumnName(i)] = statement.getUTF8String(i); 
					}
				}
				results1.push(row);
			}	
		} catch(e) {
			
		}
		
		var book_metadata = BookMetadata.getAllEx();
		
		for (var i=0; i< book_metadata.length; i++) {
			book_metadata_lookup[book_metadata[i]['id']] = true;
		}
		
		for (var i=0; i< results1.length; i++) {
			var row = results1[i];
			if (book_metadata_lookup && book_metadata_lookup.hasOwnProperty(row['bookid'])){
				results2.push(row);
			}
		}
		
		results2.sort(self.compare);
		
		config.results = results2;
		config.currpage = 1;
		
		statement.finalize();
		mDBConn.close();
		displayBookSummaryResults(results2, true);
		return results2;
		
	};
	
	var escapeTerms = function(inval) {
		inval = inval.replace(/\'/ig, "\\'");
		inval = inval.replace(/\"/ig, "\\\"");
		inval = inval.replace(/\(/ig, "\\(");
		inval = inval.replace(/\)/ig, "\\)");
		return inval;
	}
	
	var displayBookSummaryResults = function (results, init_nav) {
		var final_res = "";
		var resultCount = 0;
		var booksCount = 0;
		
		var s_index =  (config.currpage - 1) * config.items_per_page;
		var e_index =  s_index + config.items_per_page;
		if (e_index > results.length) e_index = results.length;
		
		var summarytpl = '<html:p class="search_summary">Found <html:b>"' + config.search_term + '"</html:b> in <html:b>' + results.length;
		
		if (results.length > 1) {
			summarytpl = summarytpl + '</html:b> Books. </html:p>';
		} else {
			summarytpl = summarytpl + '</html:b> Book. </html:p>';
		}
		
		for (var i=0; i< results.length; i++) {
			row = results[i];
			var booktpl = '<html:li class="book-title" id="$bookid$" onclick="javascript: IGPS.searchByBookSections(\'$bookid$\', \'$title$\')"><html:span class="result-booktitle">$title$</html:span><html:span class="result-info">$brc$</html:span><html:br/><html:span class="result-snippet">$br_snip$</html:span></html:li>'
			var bookid = row['bookid'];
			var booktitle = escapeTerms(row['title']);
			var snippet = booktpl;
			var pagecount = row['totalhits'];
			var b_snippet = row['snippet(bi)'];
			resultCount = resultCount + pagecount;
			booksCount ++;
			var snippet = booktpl;
			snippet = snippet.replace(/\$bookid\$/ig, bookid);
			snippet = snippet.replace("$title$", booktitle);
			snippet = snippet.replace("$title$", booktitle);
			snippet = snippet.replace("$brc$", pagecount);
			snippet = snippet.replace("$br_snip$", b_snippet);
			final_res = final_res + snippet;
		}
		if (final_res) { 
			final_res = final_res.replace(/&nbsp;/ig, " ");
			final_res = final_res.replace(/&/ig, " ");
			final_res = final_res.replace(/<b>/ig, "<html:b>");
			final_res = final_res.replace(/<\/b>/ig, "</html:b>");
			final_res = summarytpl + "<html:ol>" + final_res + "</html:ol>";
		} else {
			final_res = "<html:p class='search_summary'>No results found.</html:p>";
		}
		$("#search_results").html(final_res);
		if (results.length > config.items_per_page && init_nav) {
			$("#sr-pagination").pagination({
				items: results.length,
				itemsOnPage: config.items_per_page,
				cssStyle: 'compact-theme',
				onPageClick: function(pageNumber, event) {
					config.currpage = pageNumber;
					displayBookSummaryResults(config.results, false);
				}
			});
		} else {
			$("#sr-pagination").hide();
		}
	}
	var setPaginationCount = function(){
		if(Shelf.config.currentShelfType == "acfs"){
			IGPBSS.setPaginationCount();
			return false;
		}
			
		var counts = $('#resuls_per_page').val();
		config.items_per_page = counts;	
		$("#sr-pagination").pagination('updateItemsOnPage', config.items_per_page);
		if(config.results.length > config.items_per_page){
			$("#sr-pagination").show();
		} else {
			$("#sr-pagination").hide();
		}
	}
	
	var setHiliteIdList = function(hilite_ids) {
		config.hilite_ids = hilite_ids;
		config.current_hl_index  = 0;
	}
	
	var getNextHiliteTermID = function(){
		config.current_hl_index = config.current_hl_index + 1;
		if (config.current_hl_index >= config.hilite_ids.length-1){
			config.current_hl_index  = config.hilite_ids.length-1;
		}
		setSearchNavButtonStatus();
		var res = config.hilite_ids[config.current_hl_index];
		return res;
	}
	
	var getPrevHiliteTermID = function(){
		config.current_hl_index = config.current_hl_index - 1;
		if (config.current_hl_index <= 1){
			config.current_hl_index  = 1;
		}
		setSearchNavButtonStatus();
		var res = config.hilite_ids[config.current_hl_index];
		return res;
	}
	
	var loadNextResultSection = function() {
		var t_index = config.clickedIndex;
		if(Shelf.config.currentShelfType == "acfs"){
			IGPBSS.loadNextResultSection();
			return false;
		}
			
		if (t_index+1 >= config.results.length) {
			//we are at the end of results;dont do anything
			return false;
		}
		var res = config.results[t_index+1];
		var idref = res['idref'].replace(/\"/ig, "");;
		
		if (config.currBookID && idref) {
			config.clickedItem = idref;
			config.clickedIndex = t_index+1;
			AME.openBook(config.currBookID, "search");
		}
	}
	
	var loadPrevResultSection = function() {
		var t_index = config.clickedIndex;
		
		if(Shelf.config.currentShelfType == "acfs"){
			IGPBSS.loadPrevResultSection();
			return false;
		}
		
		if (t_index-1 < 0) {
			//we are at the beginning of results;dont do anything
			return false;
		}
		var res = config.results[t_index-1];
		var idref = res['idref'].replace(/\"/ig, "");;
		if (config.currBookID && idref) {
			config.clickedItem = idref;
			config.clickedIndex = t_index-1;
			AME.openBook(config.currBookID, "search");
		}
	}
	
	var setSearchNavButtonStatus = function() {
		if (config.current_hl_index <= 1 && config.clickedIndex == 0){
			$('#but_search_nav_previous').attr('disabled', true);
		} else {
			$('#but_search_nav_previous').attr('disabled', false);
		}
		if ((config.clickedIndex == config.results.length-1) && (config.current_hl_index >= config.hilite_ids.length-1)){
			$('#but_search_nav_next').attr('disabled', true);
		} else {
			$('#but_search_nav_next').attr('disabled', false);
		}
	}
	
	var clearConfig = function() {
		try{
			config.clickedItem = "";
			config.clickedIndex = "";
			config.currBookID = "";
			config.current_hl_index=0;
			config.curr_book_title="";
			//config.results.slice(0, config.results.length-1);
			config.hilite_ids.slice(0, config.hilite_ids.length-1);
		} catch(e){
		
		}
	}
	
	return {
		config: config,
		searchByBookSections: searchByBookSections,
		searchByBooks: searchByBooks,
		handleSectionClick: handleSectionClick,
		getClickedItemId: getClickedItemId,
		getSearchTerm: getSearchTerm,
		setPaginationCount: setPaginationCount,
		setHiliteIdList: setHiliteIdList,
		getNextHiliteTermID: getNextHiliteTermID,
		getPrevHiliteTermID: getPrevHiliteTermID,
		setSearchNavButtonStatus: setSearchNavButtonStatus,
		loadPrevResultSection: loadPrevResultSection,
		loadNextResultSection: loadNextResultSection,
		clearConfig: clearConfig,
	}
}();

IGPBSS = function() {
	var config = {
		items_per_page: 10,
		search_term: "",
		shelfid: "",
		currpage: 1,
		results: [],
		curr_book_title: "",
		clickedItem: "",
		clickedIndex: "",
		currBookID: "",
		hilite_ids: [],
		current_hl_index: 0
	};
	
	/*var preProcessSearchTerm = function(searchTerm) {
		var s = searchTerm;
		var sl, res=[];
		s = s.replace(/AND/ig, "");
		s = s.replace(/OR/ig, "");
		s = s.replace(/NOT/ig, "");
		s = s.replace(/\"/g, "");
		s = s.replace(/\* /g, "");
		s = s.toLowerCase();
		sl = s.split(" ");
		for (var i=0; i<sl.length; i++) {
			if(sl[i]) {
				res.push(sl[i]);
			}
		}
		return res;
	}*/
	
	var preProcessSearchTerm = function(searchterm) {
		var preProcessSearchTerm = function(searchTerm) {
			var s = searchTerm;
			s = $.trim(s);
			s = s.toLowerCase();
			if (s == "and" || s == "or" || s == "not") {
				s = "";
			}
			return s;
		}
		
		var tokenize = function(indata) {
			var res = [];
			var s = "";
			var in_quotes = false;
			for (var i=0; i < indata.length; i++) {
				if (indata[i] == "\"" && !in_quotes) {
					s = preProcessSearchTerm(s);
					if(s){
						res.push(s);
					}
					s="";
					in_quotes = true;
				} else if (indata[i] == "\"" && in_quotes) {
					if(s)res.push(s);
					s="";
					in_quotes = false;
				} else if (indata[i] == " " && !in_quotes) {
					s = preProcessSearchTerm(s);
					if(s)res.push(s);
					s="";
				} else {
					s = s + indata[i];
				}
			}
			
			s = preProcessSearchTerm(s);
			if(s) {
				res.push(s);
			}
			return res;
		}
		
		var processWildcard = function(in_tok) {
			var res = "";
			
			if (in_tok.indexOf("*") < 0) {
				//in_tok =  "\\b" + in_tok + "\\b";
				return in_tok;
			}
			
			var in_tok_list = in_tok.split(" ");
			
			for (var i=0; i < in_tok_list.length; i++) {
				var token = in_tok_list[i];
				if (token[0] == "*"){
					token = token.replace("*", "\\b\\w+");
					token = token + "\\b";
				}
				if (token[token.length-1] == "*"){
					token = token.substring(0, token.length-1) +  "\\w+\\b";
					token = "\\b" + token;
				} else {
					token = token.replace(/\*/ig, "\\w+");
					//token =  "\\b" + token + "\\b";
				}
				
				res = res + token;
			}
			
			return res;
		}
		
		var p_st = searchterm ; //preProcessSearchTerm(searchterm);
		var token_list = tokenize(p_st);
		token_list = processWildcard(token_list);
		for (var i=0; i < token_list.length; i++) {
			var mod_tok = token_list[i];
			mod_tok = processWildcard(mod_tok);
			token_list[i] = mod_tok;
		}
		
		return token_list;
	}
	
	var getClickedItemId = function() {
		return config.clickedItem;
	}
	
	var getSearchTerm = function() {
		if (!config.search_term)return "";
		var terms = preProcessSearchTerm(config.search_term);
		return terms;
	}
	
	var handleSectionClick = function(bookid, idref, res_index) {
		var openid = bookid;
		
		if (openid) {
			config.clickedItem = idref;
			config.clickedIndex = parseInt(res_index);
			config.currBookID = openid;
			AME.openBook(openid, "search");
		}
	};
	
	var displayBookDetailResults = function (results, init_nav) {
		try {
			var final_res = "";
			var bookstitle = "";
			var s_index =  (config.currpage - 1) * config.items_per_page;
			var e_index =  s_index + config.items_per_page;
			if (e_index > results.length) e_index = results.length;
			bookstitle = config.curr_book_title;
			
			var summarytpl = '<html:p class="search_summary"><html:span class="result-booktitle">' + bookstitle + '</html:span> - Found <html:b>"' + config.search_term + '"</html:b> in <html:b>' + results.length;
			
			if (results.length > 1) {
				summarytpl = summarytpl + '</html:b> sections. </html:p>';
			} else {
				summarytpl = summarytpl + '</html:b> section. </html:p>';
			}
			
			for (var i= s_index; i< e_index; i++) {
				row = results[i];
				var booktpl = '<html:li class="book-title" id="$bookid$" onclick="javascript: IGPBSS.handleSectionClick(\'$bookid$\', \'$idref$\', \'$index$\')"><html:b>$title$</html:b> <html:span class="result-snippet">$br_snip$</html:span><html:span class="result-info">$brc$</html:span></html:li>'
				var bookid = row['bookid'];
				var sec_title = row['title'];
				var snippet = booktpl;
				var pagecount = row['totalhits'];	
				var b_snippet = row['snippet(bspi)'];
				var idref = row['idref'].replace(/\"/ig, "");
				idref = escapeTerms(idref);
				
				b_snippet = b_snippet.replace(/<b>/ig, "###b###");
				b_snippet = b_snippet.replace(/<\/b>/ig, "###/b###");
				b_snippet = b_snippet.replace(/(<([^>]+)>)/ig,"");
				b_snippet = b_snippet.replace(/###b###/ig, "<b>");
				b_snippet = b_snippet.replace(/###\/b###/ig, "<\/b>");
				
				var snippet = booktpl;
				snippet = snippet.replace(/\$bookid\$/ig, bookid);
				var rep_title = sec_title;
				if (rep_title) rep_title = sec_title + ": ";
				snippet = snippet.replace("$title$", rep_title);
				snippet = snippet.replace("$idref$", idref);
				snippet = snippet.replace("$brc$", pagecount);
				snippet = snippet.replace("$br_snip$", b_snippet);
				snippet = snippet.replace("$index$", i);
				//if (config.currpage == 3)alert(snippet);
				final_res = final_res + snippet;
			}
			
			if (final_res) { 
				final_res = final_res.replace(/&#([^\s]*);/g, function(match, match2) {return String.fromCharCode(Number(match2));})
				final_res = final_res.replace(/&nbsp;/ig, " ");
				final_res = final_res.replace(/&/ig, " ");
				final_res = final_res.replace(/<b>/ig, "<html:b>");
				final_res = final_res.replace(/<\/b>/ig, "</html:b>");
				final_res = final_res.replace(/<br>/ig, "");
				final_res = summarytpl + "<html:ol>" + final_res + "</html:ol>";
				
			} else {
				final_res = "<html:p class='search_summary'>No results found.</html:p>";
			}
			
			document.getElementById('search_results').innerHTML = final_res;
			
			if (results.length > config.items_per_page && init_nav) {
				$("#sr-pagination").pagination({
					items: results.length,
					itemsOnPage: config.items_per_page,
					cssStyle: 'compact-theme',
					onPageClick: function(pageNumber, event) {
						config.currpage = pageNumber;
						displayBookDetailResults(config.results, false);
					}
				});
				$("#sr-pagination").show();
			} else if(init_nav) {
				$("#sr-pagination").hide();
			}
		} catch(e) {
			alert(e);
		}
	}
	
	var searchByBookSections = function(bookid, booktitle) {
		try {
			
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "bspi";
			var sql =  'SELECT  shelfid, bookid, title, idref, snippet(bspi), hex(matchinfo(bspi,"pcx")) FROM bspi  ';
			sql = sql +  "  WHERE content MATCH \'" + config.search_term + '\' AND bookid=\''+bookid+ '\'';
			//Now open the sqlite database as 
			var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
			var mDBConn = storageService.openDatabase(fileObj);
			var statement = mDBConn.createStatement(sql);
			var results1 = [];
			
			statement.execute();
			
			while (statement.executeStep()) {
				var row = [];				
				for (var i=0, k = statement.columnCount;i<k;i++){
					if (statement.getColumnName(i) === 'hex(matchinfo(bspi,"pcx"))') {
						var value = statement.getUTF8String(i);
						var totalhits = convertMatchInfo(value);
						row["totalhits"] = totalhits; 
						row[statement.getColumnName(i)] = value; 
					} else {
						row[statement.getColumnName(i)] = statement.getUTF8String(i); 
					}
				}
				results1.push(row);
			}
		} catch(e) {
			alert(e);
		}
		
		results1.sort(self.compare);
		
		config.results = results1;
		config.currpage = 1;
		config.curr_book_title = booktitle;
		displayBookDetailResults(results1, true);
		statement.finalize();
		mDBConn.close();
		return results1;
		
	};
	
	var convertMatchInfo = function(hex_mi) {
		try {
			var res = [];
			var strlen = hex_mi.length;
			var iter_cnt = strlen/8;
			
			dump("\n convertMatchInfo iter_cnt " + iter_cnt + "\t" + hex_mi);
			for (var i=0;i<iter_cnt;i++){
				var si = i*8;
				var ei = si + 8;
				var val = hex_mi.substr(si, 8);
				val = val.split("").reverse().join("");
				var_h = parseInt(val, 32);
				var_d = Math.round(var_h/32);
				dump("\n convertMatchInfo " + var_h + "\t" + var_d);
				res.push(var_d);
			}
			
			var phrasecnt = res[0];
			var colcnt = res[1];
			
			var totalhits = 0;
			
			for (var i=2;i<res.length;i=i+3){
				var index = i;
				var hits = res[index];
				totalhits = totalhits + hits;
				dump("\nindex " + index + "\ttotalhits " + totalhits);
			}
			
			return totalhits;
		}catch(e){
			alert(e);
			return 0;
		}
	}
	
	compare = function (a,b) {
	  if (a.totalhits > b.totalhits)
		 return -1;
	  if (a.totalhits < b.totalhits)
		return 1;
	  return 0;
	}
	
	var isBookActive = function(bookid) {
		for(var i = 0; i < Library.config.bookMetadata.length; i++){
			if("" + Library.config.bookMetadata[i]['id'] === bookid){
				if (Library.config.bookMetadata[i]['bookstate'] === "READY") {
					return true;
				}
				return false;
			}
		}
		return false;
	}
	
	var searchByBooks = function(searchterm, shelfid) {
		try {
			var book_metadata_lookup = {};
			searchterm = searchterm.replace(/'/g,"'\'");
			config.search_term = searchterm;
			config.shelfid = shelfid;
			config.currpage = 1;
			var fileObj = getFileObject(AME.config.searchDbFilePath);
			var table_name = "bsbi";
			var sql =  'SELECT  shelfid, bookid, title, snippet(bsbi), hex(matchinfo(bsbi,"pcx")) FROM bsbi';
			sql = sql +  "  WHERE content MATCH \'" + searchterm + "'" + " AND shelfid='"+shelfid+ "'";;
			
			//Now open the sqlite database as 
			var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Components.interfaces.mozIStorageService);  
			var mDBConn = storageService.openDatabase(fileObj);
			var statement = mDBConn.createStatement(sql);
			var results1 = [];
			var results2 = [];
			
			statement.execute();
			
			while (statement.executeStep()) {
				var row = [];				
				for (var i=0, k = statement.columnCount;i<k;i++){
					if (statement.getColumnName(i) === 'hex(matchinfo(bsbi,"pcx"))') {
						var value = statement.getUTF8String(i);
						var totalhits = convertMatchInfo(value);
						row["totalhits"] = totalhits; 
						row[statement.getColumnName(i)] = value; 
					} else {
						row[statement.getColumnName(i)] = statement.getUTF8String(i); 
					}
				}
				results1.push(row);
			}	
		} catch(e) {
			
		}
		var book_metadata = BookMetadata.getBSShelfBooks(shelfid);
		
		for (var i=0; i< book_metadata.length; i++) {
			book_metadata_lookup[book_metadata[i]['id']] = true;
		}
		
		for (var i=0; i< results1.length; i++) {
			var row = results1[i];
			if (book_metadata_lookup && book_metadata_lookup.hasOwnProperty(row['bookid']) && isBookActive(row['bookid'])){
				results2.push(row);
			}
		}
		
		results2.sort(self.compare);
		
		config.results = results2;
		config.currpage = 1;
		
		statement.finalize();
		mDBConn.close();
		displayBookSummaryResults(results2, true);
		return results2;
		
	};
	
	var escapeTerms = function(inval) {
		inval = inval.replace(/\'/ig, "\\'");
		inval = inval.replace(/\"/ig, "\\\"");
		inval = inval.replace(/\(/ig, "\\(");
		inval = inval.replace(/\)/ig, "\\)");
		return inval;
	}
	
	var displayBookSummaryResults = function (results, init_nav) {
		var final_res = "";
		var resultCount = 0;
		var booksCount = 0;
		
		var s_index =  (config.currpage - 1) * config.items_per_page;
		var e_index =  s_index + config.items_per_page;
		if (e_index > results.length) e_index = results.length;
		
		var summarytpl = '<html:p class="search_summary">Found <html:b>"' + config.search_term + '"</html:b> in <html:b>' + results.length;
		
		if (results.length > 1) {
			summarytpl = summarytpl + '</html:b> Books. </html:p>';
		} else {
			summarytpl = summarytpl + '</html:b> Book. </html:p>';
		}
		
		for (var i=0; i< results.length; i++) {
			row = results[i];
			var booktpl = '<html:li class="book-title" id="$bookid$" onclick="javascript: IGPBSS.searchByBookSections(\'$bookid$\', \'$title$\')"><html:span class="result-booktitle">$title$</html:span><html:span class="result-info">$brc$</html:span><html:br/><html:span class="result-snippet">$br_snip$</html:span></html:li>'
			var bookid = row['bookid'];
			var booktitle = escapeTerms(row['title']);
			var snippet = booktpl;
			var pagecount = row['totalhits'];
			var b_snippet = row['snippet(bsbi)'];
			resultCount = resultCount + pagecount;
			booksCount ++;
			var snippet = booktpl;
			snippet = snippet.replace(/\$bookid\$/ig, bookid);
			snippet = snippet.replace("$title$", booktitle);
			snippet = snippet.replace("$title$", booktitle);
			snippet = snippet.replace("$brc$", pagecount);
			snippet = snippet.replace("$br_snip$", b_snippet);
			final_res = final_res + snippet;
		}
		if (final_res) { 
			final_res = final_res.replace(/&nbsp;/ig, " ");
			final_res = final_res.replace(/&/ig, " ");
			final_res = final_res.replace(/<b>/ig, "<html:b>");
			final_res = final_res.replace(/<\/b>/ig, "</html:b>");
			final_res = summarytpl + "<html:ol>" + final_res + "</html:ol>";
		} else {
			final_res = "<html:p class='search_summary'>No results found.</html:p>";
		}
		$("#search_results").html(final_res);
		if (results.length > config.items_per_page && init_nav) {
			$("#sr-pagination").pagination({
				items: results.length,
				itemsOnPage: config.items_per_page,
				cssStyle: 'compact-theme',
				onPageClick: function(pageNumber, event) {
					config.currpage = pageNumber;
					displayBookSummaryResults(config.results, false);
				}
			});
		} else {
			$("#sr-pagination").hide();
		}
	}
	var setPaginationCount = function(){
		var counts = $('#resuls_per_page').val();
		config.items_per_page = counts;	
		$("#sr-pagination").pagination('updateItemsOnPage', config.items_per_page);
		if(config.results.length > config.items_per_page){
			$("#sr-pagination").show();
		} else {
			$("#sr-pagination").hide();
		}
	}
	
	var setHiliteIdList = function(hilite_ids) {
		config.hilite_ids = hilite_ids;
		config.current_hl_index  = 0;
	}
	
	var getNextHiliteTermID = function(){
		config.current_hl_index = config.current_hl_index + 1;
		if (config.current_hl_index >= config.hilite_ids.length-1){
			config.current_hl_index  = config.hilite_ids.length-1;
		}
		setSearchNavButtonStatus();
		var res = config.hilite_ids[config.current_hl_index];
		return res;
	}
	
	var getPrevHiliteTermID = function(){
		config.current_hl_index = config.current_hl_index - 1;
		if (config.current_hl_index <= 1){
			config.current_hl_index  = 1;
		}
		setSearchNavButtonStatus();
		var res = config.hilite_ids[config.current_hl_index];
		return res;
	}
	
	var loadNextResultSection = function() {
		var t_index = config.clickedIndex;
		if (t_index+1 >= config.results.length) {
			//we are at the end of results;dont do anything
			return false;
		}
		var res = config.results[t_index+1];
		var idref = res['idref'].replace(/\"/ig, "");;
		
		if (config.currBookID && idref) {
			config.clickedItem = idref;
			config.clickedIndex = t_index+1;
			AME.openBook(config.currBookID, "search");
		}
	}
	
	var loadPrevResultSection = function() {
		var t_index = config.clickedIndex;
		
		if (t_index-1 < 0) {
			//we are at the beginning of results;dont do anything
			return false;
		}
		var res = config.results[t_index-1];
		var idref = res['idref'].replace(/\"/ig, "");;
		if (config.currBookID && idref) {
			config.clickedItem = idref;
			config.clickedIndex = t_index-1;
			AME.openBook(config.currBookID, "search");
		}
	}
	
	var setSearchNavButtonStatus = function() {
		if (config.current_hl_index <= 1 && config.clickedIndex == 0){
			$('#but_search_nav_previous').attr('disabled', true);
		} else {
			$('#but_search_nav_previous').attr('disabled', false);
		}
		if ((config.clickedIndex == config.results.length-1) && (config.current_hl_index >= config.hilite_ids.length-1)){
			$('#but_search_nav_next').attr('disabled', true);
		} else {
			$('#but_search_nav_next').attr('disabled', false);
		}
	}
	
	var clearConfig = function() {
		try{
			config.clickedItem = "";
			config.clickedIndex = 0;
			config.currBookID = "";
			config.current_hl_index=0;
			config.curr_book_title="";
			//config.results.slice(0, config.results.length-1);
			config.hilite_ids.slice(0, config.hilite_ids.length-1);
		} catch(e){
		
		}
	}
	
	return {
		config: config,
		searchByBookSections: searchByBookSections,
		searchByBooks: searchByBooks,
		handleSectionClick: handleSectionClick,
		getClickedItemId: getClickedItemId,
		getSearchTerm: getSearchTerm,
		setPaginationCount: setPaginationCount,
		setHiliteIdList: setHiliteIdList,
		getNextHiliteTermID: getNextHiliteTermID,
		getPrevHiliteTermID: getPrevHiliteTermID,
		setSearchNavButtonStatus: setSearchNavButtonStatus,
		loadPrevResultSection: loadPrevResultSection,
		loadNextResultSection: loadNextResultSection,
		clearConfig: clearConfig,
	}
}();
